@@ -424,10 +424,6 @@ struct burn_drive *burn_drive_finish_enum(struct burn_drive *d)
 	char msg[BURN_DRIVE_ADR_LEN + 160];
 	int ret;
 
-	/* ts A60821
-   	<<< debug: for tracing calls which might use open drive fds */
-	int mmc_function_spy(struct burn_drive *d, char * text);
-
 	d->drive_role = 1; /* MMC drive */
 
 	t = burn_drive_register(d);
@@ -521,6 +517,7 @@ int burn_drive_release_fl(struct burn_drive *d, int flag)
 			d->unlock(d);
 		if ((flag & 7) == 1)
 			d->eject(d);
+		burn_drive_snooze(d, 0);
 		d->release(d);
 	}
 
@@ -909,6 +906,22 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	out->write_simulate = !!d->mdata->simulate;
 	out->c2_errors = !!d->mdata->c2_pointers;
 	out->drive = d;
+
+#ifdef Libburn_pioneer_dvr_216d_dummy_probe_wM
+
+	/* ts A91112 */
+	/* Set default block types. The call d->probe_write_modes() is quite
+	   obtrusive. It may be performed explicitely by new API call
+             burn_drive_probe_cd_write_modes().
+	*/
+	out->tao_block_types = d->block_types[BURN_WRITE_TAO] =
+					 BURN_BLOCK_MODE1 | BURN_BLOCK_RAW0;
+	out->sao_block_types = d->block_types[BURN_WRITE_SAO] = BURN_BLOCK_SAO;
+	out->raw_block_types = d->block_types[BURN_WRITE_RAW] = 0;
+	out->packet_block_types = 0;
+
+#else /* Libburn_pioneer_dvr_216d_dummy_probe_wM */
+
 	/* update available block types for burners */
 	if (out->write_dvdram || out->write_dvdr ||
 	    out->write_cdrw || out->write_cdr)
@@ -917,10 +930,37 @@ static int drive_getcaps(struct burn_drive *d, struct burn_drive_info *out)
 	out->sao_block_types = d->block_types[BURN_WRITE_SAO];
 	out->raw_block_types = d->block_types[BURN_WRITE_RAW];
 	out->packet_block_types = d->block_types[BURN_WRITE_PACKET];
+
+#endif /* ! Libburn_pioneer_dvr_216d_dummy_probe_wM */
+
 	return 1;
 }
 
 
+#ifdef Libburn_pioneer_dvr_216d_dummy_probe_wM 
+
+/* ts A91112 candidate for API */
+/* Probe available CD write modes and block types.
+*/
+int burn_drive_probe_cd_write_modes(struct burn_drive_info *dinfo)
+{
+	struct burn_drive *d = dinfo->drive;
+
+	if (d == NULL)
+		return 0;
+	if (dinfo->write_dvdram || dinfo->write_dvdr ||
+	    dinfo->write_cdrw || dinfo->write_cdr)
+		d->probe_write_modes(d);
+	dinfo->tao_block_types = d->block_types[BURN_WRITE_TAO];
+	dinfo->sao_block_types = d->block_types[BURN_WRITE_SAO];
+	dinfo->raw_block_types = d->block_types[BURN_WRITE_RAW];
+	dinfo->packet_block_types = d->block_types[BURN_WRITE_PACKET];
+	return 1;
+}
+
+#endif /* Libburn_pioneer_dvr_216d_dummy_probe_wM */
+
+
 /* ts A70907 : added parameter flag */
 /* @param flag bit0= reset global drive list */
 int burn_drive_scan_sync(struct burn_drive_info *drives[],
@@ -1279,6 +1319,7 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 	int ret = -1, fd = -1, role = 0;
 	/* divided by 512 it needs to fit into a signed long integer */
 	off_t size = ((off_t) (512 * 1024 * 1024 - 1) * (off_t) 2048);
+	off_t read_size = -1;
 	struct burn_drive *d= NULL, *regd_d;
 	struct stat stbuf;
 
@@ -1291,6 +1332,16 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 			ret = fstat(fd, &stbuf);
 		else
 			ret = stat(fname, &stbuf);
+		if (ret != -1) {
+			if (S_ISREG(stbuf.st_mode))
+				read_size = stbuf.st_size;
+			else if (S_ISBLK(stbuf.st_mode)) {
+				ret = burn_os_stdio_capacity(fname,
+								&read_size);
+				if (ret <= 0)
+					read_size = -1;  
+			}
+		}
 		if (ret == -1 || S_ISBLK(stbuf.st_mode) ||
 				S_ISREG(stbuf.st_mode)) {
 			ret = burn_os_stdio_capacity(fname, &size);
@@ -1349,6 +1400,10 @@ int burn_drive_grab_dummy(struct burn_drive_info *drive_infos[], char *fname)
 		d->current_is_supported_profile = 1;
 		d->block_types[BURN_WRITE_TAO] = BURN_BLOCK_MODE1;
 		d->block_types[BURN_WRITE_SAO] = BURN_BLOCK_SAO;
+		if (read_size >= 0)
+			/* despite its name : last valid address, not size */
+			d->media_read_capacity =
+				read_size / 2048 - !(read_size % 2048);
 		burn_drive_set_media_capacity_remaining(d, size);
 
 		/* >>> ? open file for a test ? (>>> beware of "-" = stdin) */;
@@ -2621,7 +2676,58 @@ int burn_drive_set_media_capacity_remaining(struct burn_drive *d, off_t value)
 /* ts A81215 : API */
 int burn_get_read_capacity(struct burn_drive *d, int *capacity, int flag)
 {
-	*capacity = d->media_read_capacity;
+	*capacity = d->media_read_capacity + 1;
 	return (d->media_read_capacity != 0x7fffffff);
 }
 
+
+/* ts A90903 : API */
+int burn_disc_get_media_id(struct burn_drive *d,
+	char **product_id, char **media_code1, char **media_code2,
+	char **book_type, int flag)
+{
+	int ret;
+
+	*product_id = *media_code1 = *media_code2 = *book_type = NULL;
+	if (burn_drive_get_drive_role(d) != 1)
+		return 0;
+	ret = mmc_get_media_product_id(d,
+			 product_id, media_code1, media_code2, book_type,
+			 flag & 1);
+	return ret;
+}
+
+
+/* ts A90909 : API */
+/**
+    @param valid   Replies bits which indicate the validity of other reply
+                   parameters or the state of certain CD info bits:
+                   bit0= disc_type valid
+                   bit1= disc_id valid
+                   bit2= bar_code valid
+                   bit3= disc_app_code valid
+                   bit4= Disc is unrestricted (URU bit)
+                   bit5= Disc is nominally erasable (Erasable bit)
+                         This will be set with overwriteable media which
+                         libburn normally considers to be unerasable blank.
+*/
+int burn_disc_get_cd_info(struct burn_drive *d, char disc_type[80],
+			unsigned int *disc_id, char bar_code[9], int *app_code,
+	 		int *valid)
+{
+	if (d->disc_type == 0x00) {
+		strcpy(disc_type, "CD-DA or CD-ROM");
+	} else if (d->disc_type == 0x10) {
+		strcpy(disc_type, "CD-I");
+	} else if (d->disc_type == 0x20) {
+		strcpy(disc_type, "CD-ROM XA");
+	} else {
+		strcpy(disc_type, "undefined");
+	}
+	*disc_id = d->disc_id;
+	memcpy(bar_code, d->disc_bar_code, 8);
+	bar_code[9]= 0;
+	*app_code = d->disc_app_code;
+	*valid =  d->disc_info_valid;
+	return 1;
+}
