@@ -17,7 +17,27 @@
 int mmc_function_spy(struct burn_drive *d, char * text);
 
 
-/* spc command set */
+/* START STOP UNIT as of SBC-1 and SBC-2
+   0:  Opcode 0x1B
+   1:  bit0= Immed
+       bit1-7= reserved 
+   2:  reserved
+   3:  reserved
+   4:  bit0= Start (else Stop unit)
+       bit1= Load/Eject (according to Start resp. Stop)
+       bit2-3= reserved
+       bit4-7= Power Condition
+               0= Start Valid: process Start and Load/Eject bits
+               1= assume Active state
+               2= assume Idle state
+               3= assume Stanby state
+              (5= SBC-1 only: assume Sleep state)
+               7= transfer control of power conditions to logical unit
+              10= force idle condition timer to 0
+              11= force standby condition timer to 0
+              All others are reserved.
+   5:  Control (set to 0)
+*/
 static unsigned char SBC_LOAD[] = { 0x1b, 0, 0, 0, 3, 0 };
 static unsigned char SBC_UNLOAD[] = { 0x1b, 0, 0, 0, 2, 0 };
 static unsigned char SBC_START_UNIT[] = { 0x1b, 0, 0, 0, 1, 0 };
@@ -31,11 +51,6 @@ void sbc_load(struct burn_drive *d)
 		return;
 
 	scsi_init_command(&c, SBC_LOAD, sizeof(SBC_LOAD));
-/*
-	memcpy(c.opcode, SBC_LOAD, sizeof(SBC_LOAD));
-	c.oplen = sizeof(SBC_LOAD);
-	c.page = NULL;
-*/
 	c.retry = 1;
 
 	/* ts A70921 : Had to revoke Immed because of LG GSA-4082B */
@@ -59,14 +74,7 @@ void sbc_eject(struct burn_drive *d)
 		return;
 
 	scsi_init_command(&c, SBC_UNLOAD, sizeof(SBC_UNLOAD));
-/*
-	memcpy(c.opcode, SBC_UNLOAD, sizeof(SBC_UNLOAD));
-	c.oplen = sizeof(SBC_UNLOAD);
-	c.page = NULL;
-*/
-
 	c.opcode[1] |= 1; /* ts A70918 : Immed */
-
 	c.page = NULL;
 	c.dir = NO_TRANSFER;
 	d->issue_command(d, &c);
@@ -76,8 +84,11 @@ void sbc_eject(struct burn_drive *d)
 	spc_wait_unit_attention(d, 1800, "STOP UNIT (+ EJECT)", 0);
 }
 
-/* ts A61118 : is it necessary to tell the drive to get ready for use ? */
-int sbc_start_unit(struct burn_drive *d)
+
+/* ts A91112 : Now with flag */
+/* @param flag bit0= asynchronous waiting
+*/
+int sbc_start_unit_flag(struct burn_drive *d, int flag)
 {
 	struct command c;
 	int ret;
@@ -87,17 +98,37 @@ int sbc_start_unit(struct burn_drive *d)
 
 	scsi_init_command(&c, SBC_START_UNIT, sizeof(SBC_START_UNIT));
 	c.retry = 1;
-	c.opcode[1] |= 1; /* ts A70918 : Immed */
+	c.opcode[1] |= (flag & 1); /* ts A70918 : Immed */
 	c.dir = NO_TRANSFER;
 	d->issue_command(d, &c);
 	if (c.error)
 		return 0;
-	/* ts A70918 : now asynchronous */
-	d->is_stopped = 0;
+	if (!(flag & 1))
+		return 1;
+	/* ts A70918 : asynchronous */
 	ret = spc_wait_unit_attention(d, 1800, "START UNIT", 0);
 	return ret;
 }
 
+
+int sbc_start_unit(struct burn_drive *d)
+{
+	int ret;
+
+	d->is_stopped = 0; /* no endless starting attempts */
+
+	/* Asynchronous, not to block controller by waiting */
+	ret = sbc_start_unit_flag(d, 1);
+	if (ret <= 0)
+		return ret;
+	/* Synchronous to catch Pioneer DVR-216D which is ready too early.
+	   A pending START UNIT can prevent ejecting of the tray.
+	*/
+	ret = sbc_start_unit_flag(d, 0);
+	return ret;
+}
+
+
 /* ts A90824 : Trying to reduce drive noise */
 int sbc_stop_unit(struct burn_drive *d)
 {
